@@ -12,6 +12,7 @@ import unittest
 import socket
 import sys
 import time
+import traceback
 import platform
 import moznetwork
 import xml.dom.minidom as dom
@@ -182,17 +183,19 @@ class MarionetteTestRunner(object):
 
     def __init__(self, address=None, emulator=None, emulatorBinary=None,
                  emulatorImg=None, emulator_res='480x800', homedir=None,
-                 bin=None, profile=None, autolog=False, revision=None,
+                 app=None, bin=None, profile=None, autolog=False, revision=None,
                  es_server=None, rest_server=None, logger=None,
                  testgroup="marionette", noWindow=False, logcat_dir=None,
                  xml_output=None, repeat=0, perf=False, perfserv=None,
-                 gecko_path=None, testvars=None, tree=None, device=None):
+                 gecko_path=None, testvars=None, tree=None, device=None,
+                 symbols_path=None):
         self.address = address
         self.emulator = emulator
         self.emulatorBinary = emulatorBinary
         self.emulatorImg = emulatorImg
         self.emulator_res = emulator_res
         self.homedir = homedir
+        self.app = app
         self.bin = bin
         self.profile = profile
         self.autolog = autolog
@@ -215,6 +218,7 @@ class MarionetteTestRunner(object):
         self.testvars = {}
         self.tree = tree
         self.device = device
+        self.symbols_path = symbols_path
 
         if testvars:
             if not os.path.exists(testvars):
@@ -271,8 +275,11 @@ class MarionetteTestRunner(object):
             else:
                 host = 'localhost'
                 port = 2828
-            self.marionette = Marionette(host=host, port=int(port),
-                                         bin=self.bin, profile=self.profile,
+            self.marionette = Marionette(host=host,
+                                         port=int(port),
+                                         app=self.app,
+                                         bin=self.bin,
+                                         profile=self.profile,
                                          baseurl=self.baseurl)
         elif self.address:
             host, port = self.address.split(':')
@@ -283,7 +290,8 @@ class MarionetteTestRunner(object):
                                              homedir=self.homedir,
                                              baseurl=self.baseurl,
                                              logcat_dir=self.logcat_dir,
-                                             gecko_path=self.gecko_path)
+                                             gecko_path=self.gecko_path,
+                                             symbols_path=self.symbols_path)
             else:
                 self.marionette = Marionette(host=host,
                                              port=int(port),
@@ -298,7 +306,8 @@ class MarionetteTestRunner(object):
                                          baseurl=self.baseurl,
                                          noWindow=self.noWindow,
                                          logcat_dir=self.logcat_dir,
-                                         gecko_path=self.gecko_path)
+                                         gecko_path=self.gecko_path,
+                                         symbols_path=self.symbols_path)
         else:
             raise Exception("must specify binary, address or emulator")
 
@@ -355,6 +364,11 @@ class MarionetteTestRunner(object):
         self.logger.info('passed: %d' % self.passed)
         self.logger.info('failed: %d' % self.failed)
         self.logger.info('todo: %d' % self.todo)
+        try:
+            self.marionette.check_for_crash()
+        except:
+            traceback.print_exc()
+
         self.elapsedtime = datetime.utcnow() - starttime
         if self.autolog:
             self.post_to_autolog(self.elapsedtime)
@@ -610,6 +624,9 @@ def parse_options():
                       "tests from .ini files.")
     parser.add_option('--homedir', dest='homedir', action='store',
                       help='home directory of emulator files')
+    parser.add_option('--app', dest='app', action='store',
+                      default=None,
+                      help='application to use')
     parser.add_option('--binary', dest='bin', action='store',
                       help='gecko executable to launch before running the test')
     parser.add_option('--profile', dest='profile', action='store',
@@ -636,6 +653,10 @@ def parse_options():
     parser.add_option('--tree', dest='tree', action='store',
                       default='b2g',
                       help='the tree that the revsion parameter refers to')
+    parser.add_option('--symbols-path', dest='symbols_path', action='store',
+                      default=None,
+                      help='absolute path to directory containing breakpad '
+                      'symbols, or the URL of a zip file containing symbols')
 
     options, tests = parser.parse_args()
 
@@ -677,6 +698,7 @@ def startTestRunner(runner_class, options, tests):
                           emulator_res=options.emulator_res,
                           homedir=options.homedir,
                           logcat_dir=options.logcat_dir,
+                          app=options.app,
                           bin=options.bin,
                           profile=options.profile,
                           noWindow=options.noWindow,
@@ -690,7 +712,8 @@ def startTestRunner(runner_class, options, tests):
                           perfserv=options.perfserv,
                           gecko_path=options.gecko_path,
                           testvars=options.testvars,
-                          device=options.device)
+                          device=options.device,
+                          symbols_path=options.symbols_path)
     runner.run_tests(tests, testtype=options.type)
     return runner
 
